@@ -1,7 +1,7 @@
 #
 # This file is part of the RapidMiner Python package.
 #
-# Copyright (C) 2018-2020 RapidMiner GmbH
+# Copyright (C) 2018-2021 RapidMiner GmbH
 #
 # This program is free software: you can redistribute it and/or modify it under the terms of the
 # GNU Affero General Public License as published by the Free Software Foundation, either version 3
@@ -25,6 +25,7 @@ from .serdeutils import set_metadata_without_warning
 from .utilities import ProjectException
 from .utilities import TooManyBinomialValuesError
 from .utilities import ValueConversionError
+from .connections import Connections
 
 
 
@@ -39,6 +40,7 @@ class Project():
     Class for using a project from RapidMiner Server that has been cloned locally. Use git for cloning the repository, then read and write calls can work on local resources. You need to use git commands to push changes that you make locally.
     """
 
+    __NANOSECONDS_IN_A_DAY = 86400000000000
     _RM_HDF5_EXTENSION = ".rmhdf5table"
     _RM_RMP_EXTENSION = ".rmp"
     _METADATA_TYPES = {"NOMINAL": np.int8(1),
@@ -51,9 +53,12 @@ class Project():
                        "DATE_TIME": np.int8(9),
                        "DATE": np.int8(10),
                        "TIME": np.int8(11)}
-    _LEGACY_TYPES = ["TEXT", "BINOMINAL", "FILE_PATH", "DATE", "TIME"] # POLYNOMIAL handled separatly
+    _LEGACY_TYPES = ["TEXT", "BINOMINAL", "FILE_PATH", "DATE"] # POLYNOMIAL handled separatly
     _METADATA_ROLES = ["BATCH", "CLUSTER", "ID", "LABEL", "OUTLIER", "PREDICTION", "WEIGHT"]
 
+    __RM_MISSING_DATETIME_OR_TIME = 9223372036854775807 # constant value is coming from com.rapidminer.storage.hdf5.ExampleSetHdf5Writer.java#writeDateData/writeTimeData as of Long.MAX_VALUE/TimeColumn.MISSING_VALUE
+    __PANDAS_MISSING_DATE = pd.to_numeric(pd.Series([datetime.utcfromtimestamp(0), None]), downcast="integer")[1]
+
     def __init__(self, path="."):
         """
         Initializes a reference to a locally cloned project. You need to clone a project from RapidMiner Server first (e.g. via git commands) to be able to use the methods of this instance.
@@ -61,7 +66,6 @@ class Project():
         :param path: path to the local project repository root folder. It can be a relative path from the current working directory or an absolute path, . The default value points to the working directory.
         """
         if not os.path.exists(path):
-            dirname = os.path.dirname(path)
             if path == "":
                 msg_dir_part = "in the current directory"
             else:
@@ -95,12 +99,18 @@ class Project():
             path = path + Project._RM_HDF5_EXTENSION
         Project.__write_data_safe(df, path)
 
+    def get_connections(self):
+        """
+        Returns the connection in that this project contains.
+        """
+        return Connections(self.path)
         
 #####################
 # Private functions #
 #####################
 
-    __from_ts = np.vectorize(datetime.utcfromtimestamp)
+    __from_ts_nanos = np.vectorize(lambda x: datetime.utcfromtimestamp(x/1e9) if x != Project.__RM_MISSING_DATETIME_OR_TIME else None)
+    __from_ts_seconds_and_nanos = np.vectorize(lambda x,y: datetime.utcfromtimestamp(x+y/1e9) if x != Project.__RM_MISSING_DATETIME_OR_TIME else None)
     __hyp5_string_dtype = h5py.string_dtype()
     __h5py_reference_dtype = h5py.special_dtype(ref=h5py.Reference)
     
@@ -113,7 +123,9 @@ class Project():
             else:
                 return x
         elif typestr in ('Date-Time', 'Date'):
-            return Project.__from_ts(x[:])
+            return Project.__from_ts_seconds_and_nanos(x[:],0)
+        elif typestr == "Time":
+            return Project.__from_ts_nanos(x[:])
         else:
             return x
 
@@ -123,7 +135,7 @@ class Project():
         if "additional" in r.attrs and r.attrs.get("type") == "Date-Time":
             additional = r.attrs.get('additional')
             additional = f[additional]
-            return Project.__from_ts(r[:] + additional[:] / 1e9)
+            return Project.__from_ts_seconds_and_nanos(r[:], additional[:])
         if mapping is None:
             return Project.__get_numerical(r, decode(r.attrs.get('type')))
         if isinstance(mapping,h5py.Reference):
@@ -141,8 +153,10 @@ class Project():
                 if Project._METADATA_TYPES[k] == hdf_attrs.get("legacy_type"):
                     return k.lower()
         hdf_type = decode(hdf_attrs.get("type"))
-        if hdf_type in ("Date-Time", "Date", "Time"):
+        if hdf_type in ("Date-Time", "Date"):
             meta_type = "date_time"
+        elif hdf_type == "Time":
+            meta_type = "time"
         elif df_kind_char in ('i', 'u'):
             meta_type = 'integer'
         elif df_kind_char in ('f'):
@@ -211,19 +225,25 @@ class Project():
             cat = column.astype("category").cat
             dset = f.create_dataset(shortname, data = pd.to_numeric(cat.codes.apply(lambda x:x+1), downcast='integer'))
             mappingname = 'd'+str(index)
-            mappingvals = cat.categories.values.astype(np.object)
+            mappingvals = cat.categories.values.astype(object)
             replacement = "NULL"
             while replacement in mappingvals:
                 replacement = "\x00" + replacement
-            mappingvals = np.concatenate([[replacement],mappingvals])
+            mappingvals = np.concatenate([[replacement], mappingvals])
             if desired_type == "BINOMINAL" and len(mappingvals) > 3:
                 raise TooManyBinomialValuesError("Column '%s' marked as binomial column in rm_metadata attribute, but has more there is more then two distinct values present.", )
             if len(mappingvals) <= 3:
                 dset.attrs["dictionary"] = [str(v) for v in mappingvals]
-                # positive index may change after read and write
-                dset.attrs['positive_index'] = np.int8(len(mappingvals) - 1)
+                if desired_type == "BINOMINAL":
+                    # positive index may change after read and write
+                    dset.attrs['positive_index'] = np.int8(len(mappingvals) - 1)
             else:
-                mset = f.create_dataset(mappingname, data=mappingvals, dtype=Project.__hyp5_string_dtype)
+                mset = f.create_dataset(mappingname, (len(mappingvals), ), dtype=Project.__hyp5_string_dtype)
+                try:
+                    mset[()] = mappingvals
+                except TypeError:
+                    # fixes error: TypeError: Can't implicitly convert non-string objects to strings (caused by some Python environments)
+                    mset[()] = [str(v) for v in mappingvals]
                 dset.attrs.create("dictionary", mset.ref, dtype=Project.__h5py_reference_dtype)
             dset.attrs['type'] = "Nominal"
         elif desired_type == "INTEGER":
@@ -231,14 +251,22 @@ class Project():
             dset.attrs['type'] = "Integer"
         elif desired_type in ["DATE", "TIME", "DATE_TIME"]:
             if column.dtype.kind == "M":
-                nanoseconds = column.dt.tz_localize(None).astype("int64")
+                nanoseconds = pd.to_numeric(column.dt.tz_localize(None), downcast="integer")
             else:
-                nanoseconds = column.astype("int64")
-            dset = f.create_dataset(shortname, data = nanoseconds // int(1e9))
-            additionalname = shortname+"a"
-            aset = f.create_dataset(additionalname, data = nanoseconds % int(1e9))
-            dset.attrs.create("additional", aset.ref, dtype=Project.__h5py_reference_dtype)
-            dset.attrs['type'] = "Date-Time"
+                nanoseconds = pd.to_numeric(column, downcast="integer")
+            if desired_type == "TIME":
+                missing = nanoseconds==Project.__PANDAS_MISSING_DATE
+                nanoseconds.loc[missing] = Project.__RM_MISSING_DATETIME_OR_TIME
+                nanoseconds.loc[~missing] = nanoseconds.loc[~missing] % int(Project.__NANOSECONDS_IN_A_DAY)
+                dset = f.create_dataset(shortname, data = nanoseconds)
+                dset.attrs['type'] = "Time"
+            else:
+                seconds = nanoseconds.apply(lambda x: x // int(1e9) if x != Project.__PANDAS_MISSING_DATE else Project.__RM_MISSING_DATETIME_OR_TIME)
+                dset = f.create_dataset(shortname, data = seconds )
+                additionalname = shortname+"a"
+                aset = f.create_dataset(additionalname, data = (nanoseconds % int(1e9)).astype("int32"))
+                dset.attrs.create("additional", aset.ref, dtype=Project.__h5py_reference_dtype)
+                dset.attrs['type'] = "Date-Time"
         else:
             dset = f.create_dataset(shortname, data = column.astype("float64"))
             dset.attrs['type'] = "Real"
