@@ -1,7 +1,7 @@
 #
 # This file is part of the RapidMiner Python package.
 #
-# Copyright (C) 2018-2020 RapidMiner GmbH
+# Copyright (C) 2018-2021 RapidMiner GmbH
 #
 # This program is free software: you can redistribute it and/or modify it under the terms of the
 # GNU Affero General Public License as published by the Free Software Foundation, either version 3
@@ -25,12 +25,14 @@ try:
 except:
     import pickle
 from time import sleep
+from .connections import Connections
 from .connector import Connector
 from .utilities import ServerException
 from .utilities import GeneralException
 from .utilities import extract_json
 from .utilities import Version
 from .utilities import VersionException
+from .utilities import _is_docker_based_deployment
 from .resources import RepositoryLocation, ProjectLocation
 from .serdeutils import read_example_set, write_example_set, is_file_object
 from .project import Project
@@ -136,6 +138,8 @@ them access to the process created by this operation."""
         """
         super(Server, self).__init__(logger, loglevel)
         # URL of the Rapidminer Server
+        if url.endswith("/"):
+            url = url[:-1]
         self.server_url = url
         if not _is_docker_based_deployment():
             # RapidMiner Server Username
@@ -197,7 +201,7 @@ them access to the process created by this operation."""
                 this_project = inp.project
                 inp = inp.path
             elif not isinstance(inp, str):
-                raise ServerException("Input path should be 'str' or 'rapidminer.RepositoryLocation object, not '" + str(type(inp)) + "'.")
+                raise ServerException("Input path should be 'str' or 'rapidminer.RepositoryLocation or 'rapidminer.ProjectLocation object, not '" + str(type(inp)) + "'.")
             if this_project:
                 resources.append(self.__read_project(this_project, inp))
             else:
@@ -343,7 +347,7 @@ them access to the process created by this operation."""
 
     def get_projects(self):
         """
-        Gets information of the available projects in the Server instance.
+        Gets information of the available projects in the AI Hub instance.
 
         :return: a JSON array of objects representing each repository with its properties
         """
@@ -351,12 +355,66 @@ them access to the process created by this operation."""
         r = self.__send_request(partial(requests.get, get_url),
                                 lambda s: "Failed to get projects, status: " + str(s))
         return r.json()
+    
+    def get_connections(self):
+        """
+        Read the connections from the AI Hub repository.
+        
+        :return: Connections object listing connections from the AI Hub repository. Note that values of encrypted fields are not available (values will be None). Use AI Hub Vault to securely store and retrieve these values instead  
+        """
+        return Connections(path=None, server=self)
         
         
 #####################
 # Private functions #
 #####################
 
+    def _get_vault_info(self, location):
+        """
+        Load all server vault entries for a repository location.
+        
+        :param location: the location to read the vault information for. Note that there is a different syntax depending on the target repository type. For Projects, use git://reponame.git/Connections/My Connection.conninfo. For AI Hub repository, use /Connections/My Connection
+        """
+        if isinstance(location, ProjectLocation):
+            location = location.to_string(with_prefix=True)
+        elif not isinstance(location, str):
+            raise ServerException("Location must be a 'str' or 'rapidminer.ProjectLocation object, not '" + str(type(inp)) + "'.")
+        get_url = self.server_url + "/executions/connections/vault?location=" + location
+        r = self.__send_request(partial(requests.get, get_url),
+                                lambda s: "Failed to get vault info, status: " + str(s))
+        return r.json()
+
+    def _get_project_info(self, project_name):
+        """
+        Read the information for a project from AI Hub.
+        
+        :param project_name: specifies the project
+        :return: info in JSON format for the project
+        """
+        get_url = self.server_url + "/executions/repositories/" + project_name
+        r = self.__send_request(partial(requests.get, get_url),
+                                lambda s: "Failed to get project info" 
+                                + (": No project exists with the name '" + project_name + "', provide a valid project name" 
+                                   if s == 404 else ", status: " + str(s)))
+        return r.json()
+
+    def _get_connections_info(self):
+        """
+        Read the connections from the AI Hub repository.
+        
+        :return: connections in JSON format
+        """
+        get_url = self.server_url + "/api/rest/repository/connections"
+        r = self.__send_request(partial(requests.get, get_url),
+                                lambda s: "Failed to get connections list, status: " + str(s))
+        return r.json()
+
+    def _read_connection_info(self, location):
+        get_url = self.server_url + "/executions/connections/detail?location=" + location
+        r = self.__send_request(partial(requests.get, get_url),
+                                lambda s: "Failed to get connection details, status: " + str(s))
+        return r.json()
+
     def __username(self):
         if _is_docker_based_deployment():
             if "JUPYTERHUB_USER" in os.environ:
@@ -374,7 +432,7 @@ them access to the process created by this operation."""
         self.log("Getting up-to-date access token from: " + api_url, level=logging.DEBUG)
         r = requests.get(api_url, headers={'Authorization': 'token %s' % api_token})
         r.raise_for_status()
-        rms_jwt = r.json()['auth_state']['rms_jwt_idToken']
+        jwt = r.json()['auth_state']['rms_jwt_idToken']
         # Bearer Authorization header
         return { 'Authorization' : 'Bearer %s' %  jwt }
 
@@ -666,7 +724,7 @@ them access to the process created by this operation."""
                         obj = pickle.load(io.BytesIO(base64.b64decode(content)))
                     except Exception as exc:
                         raise GeneralException("Error while trying to load pickled object (note that Python 2 objects may not be readable): " + str(exc))
-                    resources.append(obj)
+                    return obj
                 elif ext == 'fo':
                     return io.BytesIO(base64.b64decode(content.encode("utf-8"))) # reads the file to memory
             except KeyError as e:
@@ -692,6 +750,3 @@ them access to the process created by this operation."""
             else:
                 raise e
         return r.text
-
-def _is_docker_based_deployment():
-    return all([var in os.environ for var in ["JUPYTERHUB_API_TOKEN", "JUPYTERHUB_API_URL", "JUPYTERHUB_USER"]])
