@@ -1,7 +1,7 @@
 #
 # This file is part of the RapidMiner Python package.
 #
-# Copyright (C) 2018-2020 RapidMiner GmbH
+# Copyright (C) 2018-2021 RapidMiner GmbH
 #
 # This program is free software: you can redistribute it and/or modify it under the terms of the
 # GNU Affero General Public License as published by the Free Software Foundation, either version 3
@@ -16,7 +16,6 @@
 #
 import os
 import sys
-import pkgutil
 
 if sys.version_info.major > 2:
     def __open__(file, mode):
@@ -70,26 +69,18 @@ def extract_json(res):
         except:
             return {}
         if "error" in response:
-            str = ""
+            s = ""
             error = response["error"]
             if "type" in error:
-                str += error["type"] + ": "
+                s += error["type"] + ": "
             if "message" in error:
-                str += error["message"]
-            if str == "":
-                str = "Unkown error: " + str(response)
+                s += error["message"]
+            if s == "":
+                s = "Unkown error: " + str(response)
             raise ServerException(str)
     return response
 
 
-def put_docker_notebook_start(path, template_type="long"):
-    if template_type not in ["short", "long"]:
-        raise ValueError("Template_type should be 'long' or 'short' (" + str(template_type) + " is provided).")
-    data = pkgutil.get_data(__name__, "notebooks/server_docker_start_" + template_type + ".ipynb")
-    with open(path, "wb") as outf:
-        outf.write(data)
-
-
 class Version:
     def __init__(self, version):
         v = version.split(".")
@@ -105,3 +96,6 @@ class Version:
             elif selflist[i] < otherlist[i]:
                 return False
         return True
+
+def _is_docker_based_deployment():
+    return all([var in os.environ for var in ["JUPYTERHUB_API_TOKEN", "JUPYTERHUB_API_URL", "JUPYTERHUB_USER"]])
